/**
 * Fetchable addresses the reality that most of our resources exist in three
 * distinct possible states.
 * - not loaded: it may or may not exist, but I don't have a copy of it
 * - not found: I've asked and know definitively that this does not exist
 * - value present
 */
export type Fetchable<T> = 'loading' | 'not loaded' | 'not found' | T;

export function hasValue<T>(f: Fetchable<T>): f is T {
  return f !== 'not found' && f !== 'not loaded' && f !== 'loading';
}

export function valueOf<T>(f: Fetchable<T>): T | null {
  return !hasValue(f) ? null : f;
}

export function withValue<T, V>(f: Fetchable<T>, fun: (val: T) => V): V | null {
  return !hasValue(f) ? null : fun(f);
}

export function use<T, V>(f: Fetchable<T>, fun: (val: T) => V): Fetchable<V> {
  if (!hasValue(f)) {
    return f;
  } else {
    return fun(f);
  }
}


